#!/usr/bin/env python

"""
	--------------------------------------------------------------------------
	The SRD Software License, Version 1.0

	Copyright (C) 2000-2004 Systemic Realtime Design, LLC.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are
	met:

	1.	Redistributions of source code must retain the above copyright notice,
		this list of conditions and the following disclaimer.

	2.	Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.

	3.	The end-user documentation included with the redistribution, if any,
		must include the following acknowledgment: "This product includes
		software developed by Systemic Realtime Design, LLC."
		Alternately, this acknowledgment may appear in the software itself,
		if and wherever such third-party acknowledgments normally appear.


	THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
	SYSTEMIC REALTIME DESIGN, LLC. BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	--------------------------------------------------------------------------

						  Developed by
				  Systemic Realtime Design, LLC.
						www.SysRTime.com

	--------------------------------------------------------------------------

    This Python script reads a header file expecting to find
    a line of the form:
        #define VBUILD N
    then increments N and writes the file back
"""


import string
import time
import sys

VERSION_FNAME = sys.argv[1] 
BUILD_TOKEN = 'VBUILD'


# make time string
try:
	timestr = time.strftime("%a %d-%b-%y %X %Z", time.localtime(time.time()))
except:
	timestr = '<error>'




# Create the file if nonexistant

def create_new_version_file(fname):
	fn = open(fname, "w")
	fn.write('/*\n')
	fn.write(' *\tThis file was automatically created by: "')
	fn.write(sys.argv[0])
	fn.write('"\n *\ton ')
	fn.write(timestr)
	fn.write('\n */\n\n')
	fn.write('#ifndef VERSION_H\n')
	fn.write('#define VERSION_H\n\n')
	fn.write('#define VMAJOR 0\n')
	fn.write('#define VMINOR 1\n')
	fn.write('#define VBUILD 1\n\n')
	fn.write('#endif\n')
	fn.close()



def main():

	# try to open the file
	try:
		fn = open(VERSION_FNAME, 'r+')
	except:
		create_new_version_file(VERSION_FNAME)
		sys.exit(0)
		

	# Read in file
	lines = fn.readlines()

	# Increment the Build number
	for i in xrange(len(lines)):
		fields = string.split(lines[i])
		if (len(fields) == 3 and fields[0] == '#define' and fields[1] == BUILD_TOKEN):
			fields[2] = int(fields[2]) + 1
			lines[i] = '%s %s %d\n' % tuple(fields)
			break

	# rewind
	fn.seek(0)

	# write a header indicating we updated it
#	fn.write('/*\n')
#	fn.write(' *\tThis file was automatically updated by: "')
#	fn.write(sys.argv[0])
#	fn.write('"\n *\ton ')
#	fn.write(timestr)
#	fn.write('\n */\n\n')


	# write the file back and close
	fn.writelines(lines)
	fn.close()


if __name__ == '__main__':
	main()
